@@ -31,15 +31,16 @@
 # SPDX-License-Identifier: Apache-2.0
 # ============================================================================
 #
-from pathlib import Path
-from typing import Dict, Union, Optional as Nullable, List, Iterable, Generator, Tuple, Any as typing_Any
+from os.path import relpath as path_relpath
+from pathlib import Path as pathlib_Path
+from typing import Dict, Union, Optional as Nullable, List, Iterable, Generator, Tuple, Any as typing_Any, Type
 
 from pySVModel import VerilogVersion, SystemVerilogVersion
 from pyVHDLModel import VHDLVersion
 from pydecor import export
 
 
-__version__ = "0.2.0"
+__version__ = "0.3.1"
 
 
 @export
@@ -47,6 +48,17 @@ class Attribute:
 	KEY: str
 	VALUE_TYPE: typing_Any
 
+	@staticmethod
+	def resolve(obj: typing_Any, key: Type['Attribute']):
+		if isinstance(obj, File):
+			return obj._fileSet[key]
+		elif isinstance(obj, FileSet):
+			return obj._design[key]
+		elif isinstance(obj, Design):
+			return obj._project[key]
+		else:
+			raise Exception("Resolution error")
+
 
 @export
 class FileType(type):
@@ -60,11 +72,11 @@ class FileType(type):
 	FileTypes: Dict[str, 'FileType'] = {}     #: Dictionary of all classes of type :class:`FileType` or derived variants
 	Any: 'FileType'
 
-	def __init__(cls, name: str, bases: Tuple[type, ...], dict: Dict[str, typing_Any], **kwargs):
-		super().__init__(name, bases, dict, **kwargs)
+	def __init__(cls, name: str, bases: Tuple[type, ...], dictionary: Dict[str, typing_Any], **kwargs):
+		super().__init__(name, bases, dictionary, **kwargs)
 		cls.Any = cls
 
-	def __new__(cls, className, baseClasses, classMembers: dict):
+	def __new__(cls, className, baseClasses, classMembers: Dict):
 		fileType = super().__new__(cls, className, baseClasses, classMembers)
 		cls.FileTypes[className] = fileType
 		return fileType
@@ -72,7 +84,8 @@ class FileType(type):
 	def __getattr__(cls, item) -> 'FileType':
 		if item[:2] != "__" and item[-2:] != "__":
 			return cls.FileTypes[item]
-#		raise RuntimeError()
+		else:
+			return super().__getattribute__(item)
 
 	def __contains__(cls, item) -> bool:
 		return issubclass(item, cls)
@@ -94,16 +107,16 @@ class File(metaclass=FileType):
 	:arg fileSet: Fileset the file is associated with.
 	"""
 
-	_path:       Path
+	_path:       pathlib_Path
 	_fileType:   'FileType'
 	_project:    Nullable['Project']
 	_design:     Nullable['Design']
 	_fileSet:    Nullable['FileSet']
-	_attributes: Dict[Attribute, typing_Any] = {}
+	_attributes: Dict[Type[Attribute], typing_Any]
 
 	def __init__(
 		self,
-		path: Path,
+		path: pathlib_Path,
 		project: 'Project' = None,
 		design: 'Design' = None,
 		fileSet: 'FileSet' = None
@@ -132,16 +145,27 @@ class File(metaclass=FileType):
 			self._design =  None
 			self._fileSet = None
 
+		self._attributes = {}
+		self._registerAttributes()
+
+	def _registerAttributes(self):
+		pass
+
 	@property
 	def FileType(self) -> 'FileType':
+		"""Read-only property to return the file type of this file."""
 		return self._fileType
 
 	@property
-	def Path(self) -> Path:
+	def Path(self) -> pathlib_Path:
+		"""Read-only property returning the path of this file."""
 		return self._path
 
+	# TODO: setter?
+
 	@property
-	def ResolvedPath(self) -> Path:
+	def ResolvedPath(self) -> pathlib_Path:
+		"""Read-only property returning the resolved path of this file."""
 		if self._path.is_absolute():
 			return self._path.resolve()
 		elif self._fileSet is not None:
@@ -150,13 +174,15 @@ class File(metaclass=FileType):
 			if path.is_absolute():
 				return path
 			else:
-				return path.relative_to(Path.cwd())
+				# WORKAROUND: https://stackoverflow.com/questions/67452690/pathlib-path-relative-to-vs-os-path-relpath
+				return pathlib_Path(path_relpath(path, pathlib_Path.cwd()))
 		else:
 			# TODO: message and exception type
 			raise Exception("")
 
 	@property
 	def Project(self) -> Nullable['Project']:
+		"""Property setting or returning the project this file is used in."""
 		return self._project
 
 	@Project.setter
@@ -168,6 +194,7 @@ class File(metaclass=FileType):
 
 	@property
 	def Design(self) -> Nullable['Design']:
+		"""Property setting or returning the design this file is used in."""
 		return self._design
 
 	@Design.setter
@@ -184,6 +211,7 @@ class File(metaclass=FileType):
 
 	@property
 	def FileSet(self) -> Nullable['FileSet']:
+		"""Property setting or returning the fileset this file is used in."""
 		return self._fileSet
 
 	@FileSet.setter
@@ -191,13 +219,38 @@ class File(metaclass=FileType):
 		self._fileSet = value
 		value._files.append(self)
 
-	def __getitem__(self, key: Attribute):
+	def Validate(self):
+		"""Validate this file."""
+		if self._path is None:
+			raise Exception("Validation: File has no path.")
+		try:
+			path = self.ResolvedPath
+		except Exception as ex:
+			raise Exception(f"Validation: File '{self._path}' could not compute resolved path.") from ex
+		if not path.exists():
+			raise Exception(f"Validation: File '{self._path}' (={path}) does not exist.")
+		if not path.is_file():
+			raise Exception(f"Validation: File '{self._path}' (={path}) is not a file.")
+
+		if self._fileSet is None:
+			raise Exception(f"Validation: File '{self._path}' has no fileset.")
+		if self._design is None:
+			raise Exception(f"Validation: File '{self._path}' has no design.")
+		if self._project is None:
+			raise Exception(f"Validation: File '{self._path}' has no project.")
+
+	def __getitem__(self, key: Type[Attribute]):
+		"""Index access for returning attributes on this file."""
+		if not issubclass(key, Attribute):
+			raise TypeError("Parameter 'key' is not an 'Attribute'.")
+
 		try:
 			return self._attributes[key]
 		except KeyError:
-			return self._fileSet[key]
+			return key.resolve(self, key)
 
-	def __setitem__(self, key: Attribute, value: typing_Any):
+	def __setitem__(self, key: Type[Attribute], value: typing_Any):
+		"""Index access for setting attributes on this file."""
 		x = key.VALUE_TYPE
 		self._attributes[key] = value
 
@@ -285,22 +338,50 @@ class EDIFNetlistFile(NetlistFile):
 	"""Netlist file in EDIF (Electronic Design Interchange Format)."""
 
 
+@export
+class TCLSourceFile(SourceFile, TCLContent):
+	"""A TCL source file."""
+
+
 @export
 class VHDLSourceFile(HDLSourceFile, HumanReadableContent):
-	"""A VHDL source file (of any language version)."""
+	"""
+	A VHDL source file (of any language version).
+
+	:arg path:        Relative or absolute path to the file.
+	:arg vhdlLibrary: VHDLLibrary this VHDL source file is associated wih.
+	:arg vhdlVersion: VHDLVersion this VHDL source file is associated wih.
+	:arg project:     Project the file is associated with.
+	:arg design:      Design the file is associated with.
+	:arg fileSet:     Fileset the file is associated with.
+	"""
 
 	_vhdlLibrary: 'VHDLLibrary'
 	_vhdlVersion: VHDLVersion
 
-	def __init__(self, path: Path, vhdlLibrary: Union[str, 'VHDLLibrary'] = None, vhdlVersion: VHDLVersion = None, project: 'Project' = None, design: 'Design' = None, fileSet: 'FileSet' = None):
+	def __init__(self, path: pathlib_Path, vhdlLibrary: Union[str, 'VHDLLibrary'] = None, vhdlVersion: VHDLVersion = None, project: 'Project' = None, design: 'Design' = None, fileSet: 'FileSet' = None):
 		super().__init__(path, project, design, fileSet)
 
 		# TODO: handle if vhdlLibrary is a string
 		self._vhdlLibrary = vhdlLibrary
 		self._vhdlVersion = vhdlVersion
 
+	def Validate(self):
+		"""Validate this VHDL source file."""
+		super().Validate()
+
+		try:
+			_ = self.VHDLLibrary
+		except Exception as ex:
+			raise Exception(f"Validation: VHDLSourceFile '{self._path}' (={self.ResolvedPath}) has no VHDLLibrary assigned.") from ex
+		try:
+			_ = self.VHDLVersion
+		except Exception as ex:
+			raise Exception(f"Validation: VHDLSourceFile '{self._path}' (={self.ResolvedPath}) has no VHDLVersion assigned.") from ex
+
 	@property
 	def VHDLLibrary(self) -> 'VHDLLibrary':
+		"""Property setting or returning the VHDL library this VHDL source file is used in."""
 		if self._vhdlLibrary is not None:
 			return self._vhdlLibrary
 		elif self._fileSet is not None:
@@ -315,6 +396,7 @@ class VHDLSourceFile(HDLSourceFile, HumanReadableContent):
 
 	@property
 	def VHDLVersion(self) -> VHDLVersion:
+		"""Property setting or returning the VHDL version this VHDL source file is used in."""
 		if self._vhdlVersion is not None:
 			return self._vhdlVersion
 		elif self._fileSet is not None:
@@ -333,13 +415,14 @@ class VerilogSourceFile(HDLSourceFile, HumanReadableContent):
 
 	_verilogVersion: VerilogVersion
 
-	def __init__(self, path: Path, verilogVersion: VerilogVersion = None, project: 'Project' = None, design: 'Design' = None, fileSet: 'FileSet' = None):
+	def __init__(self, path: pathlib_Path, verilogVersion: VerilogVersion = None, project: 'Project' = None, design: 'Design' = None, fileSet: 'FileSet' = None):
 		super().__init__(path, project, design, fileSet)
 
 		self._verilogVersion = verilogVersion
 
 	@property
 	def VerilogVersion(self) -> VerilogVersion:
+		"""Property setting or returning the Verilog version this Verilog source file is used in."""
 		if self._verilogVersion is not None:
 			return self._verilogVersion
 		elif self._fileSet is not None:
@@ -358,13 +441,14 @@ class SystemVerilogSourceFile(HDLSourceFile, HumanReadableContent):
 
 	_svVersion: SystemVerilogVersion
 
-	def __init__(self, path: Path, svVersion: SystemVerilogVersion = None, project: 'Project' = None, design: 'Design' = None, fileSet: 'FileSet' = None):
+	def __init__(self, path: pathlib_Path, svVersion: SystemVerilogVersion = None, project: 'Project' = None, design: 'Design' = None, fileSet: 'FileSet' = None):
 		super().__init__(path, project, design, fileSet)
 
 		self._svVersion = svVersion
 
 	@property
 	def SVVersion(self) -> SystemVerilogVersion:
+		"""Property setting or returning the SystemVerilog version this SystemVerilog source file is used in."""
 		if self._svVersion is not None:
 			return self._svVersion
 		elif self._fileSet is not None:
@@ -459,9 +543,11 @@ class FileSet:
 	fileset.
 
 	:arg name:            Name of this fileset.
+	:arg topLevel:        Name of the fileset's toplevel.
 	:arg directory:       Path of this fileset (absolute or relative to a parent fileset or design).
 	:arg project:         Project the file is associated with.
 	:arg design:          Design the file is associated with.
+	:arg parent:          Parent fileset if this fileset is nested.
 	:arg vhdlLibrary:     Default VHDL library for files in this fileset, if not specified for the file itself.
 	:arg vhdlVersion:     Default VHDL version for files in this fileset, if not specified for the file itself.
 	:arg verilogVersion:  Default Verilog version for files in this fileset, if not specified for the file itself.
@@ -472,11 +558,11 @@ class FileSet:
 	_topLevel:        Nullable[str]
 	_project:         Nullable['Project']
 	_design:          Nullable['Design']
-	_directory:       Nullable[Path]
+	_directory:       pathlib_Path
 	_parent:          Nullable['FileSet']
 	_fileSets:        Dict[str, 'FileSet']
 	_files:           List[File]
-	_attributes:      Dict[Attribute, typing_Any]
+	_attributes:      Dict[Type[Attribute], typing_Any]
 	_vhdlLibraries:   Dict[str, 'VHDLLibrary']
 	_vhdlLibrary:     'VHDLLibrary'
 	_vhdlVersion:     VHDLVersion
@@ -487,7 +573,7 @@ class FileSet:
 		self,
 		name: str,
 		topLevel: str = None,
-		directory: Path = Path("."),
+		directory: pathlib_Path = pathlib_Path("."),
 		project: 'Project' = None,
 		design: 'Design' = None,
 		parent: Nullable['FileSet'] = None,
@@ -526,6 +612,7 @@ class FileSet:
 
 	@property
 	def Name(self) -> str:
+		"""Property setting or returning the fileset's name."""
 		return self._name
 
 	@Name.setter
@@ -534,6 +621,7 @@ class FileSet:
 
 	@property
 	def TopLevel(self) -> str:
+		"""Property setting or returning the fileset's toplevel."""
 		return self._topLevel
 
 	@TopLevel.setter
@@ -542,6 +630,7 @@ class FileSet:
 
 	@property
 	def Project(self) -> Nullable['Project']:
+		"""Property setting or returning the project this fileset is used in."""
 		return self._project
 
 	@Project.setter
@@ -550,7 +639,15 @@ class FileSet:
 
 	@property
 	def Design(self) -> Nullable['Design']:
-		return self._design
+		"""Property setting or returning the design this fileset is used in."""
+		if self._design is not None:
+			return self._design
+		elif self._parent is not None:
+			return self._parent.Design
+		else:
+			return None
+			# TODO: raise exception instead
+			# QUESTION: how to handle if design and parent is set?
 
 	@Design.setter
 	def Design(self, value: 'Design') -> None:
@@ -561,15 +658,17 @@ class FileSet:
 			raise Exception("The design's project is not identical to the already assigned project.")
 
 	@property
-	def Directory(self) -> Path:
+	def Directory(self) -> pathlib_Path:
+		"""Property setting or returning the directory this fileset is located in."""
 		return self._directory
 
 	@Directory.setter
-	def Directory(self, value: Path) -> None:
+	def Directory(self, value: pathlib_Path) -> None:
 		self._directory = value
 
 	@property
-	def ResolvedPath(self) -> Path:
+	def ResolvedPath(self) -> pathlib_Path:
+		"""Read-only property returning the resolved path of this fileset."""
 		if self._directory.is_absolute():
 			return self._directory.resolve()
 		else:
@@ -577,6 +676,8 @@ class FileSet:
 				directory = self._parent.ResolvedPath
 			elif self._design is not None:
 				directory = self._design.ResolvedPath
+			elif self._project is not None:
+				directory = self._project.ResolvedPath
 			else:
 				# TODO: message and exception type
 				raise Exception("")
@@ -585,35 +686,51 @@ class FileSet:
 			if directory.is_absolute():
 				return directory
 			else:
-				return directory.relative_to(Path.cwd())
+				# WORKAROUND: https://stackoverflow.com/questions/67452690/pathlib-path-relative-to-vs-os-path-relpath
+				return pathlib_Path(path_relpath(directory, pathlib_Path.cwd()))
 
 	@property
 	def Parent(self) -> Nullable['FileSet']:
+		"""Property setting or returning the parent fileset this fileset is used in."""
 		return self._parent
 
 	@Parent.setter
 	def Parent(self, value: 'FileSet') -> None:
 		self._parent = value
+		value._fileSets[self._name] = self
+		# TODO: check it it already exists
+		# QUESTION: make an Add fileset method?
 
 	@property
 	def FileSets(self) -> Dict[str, 'FileSet']:
+		"""Read-only property returning the a dictionary of sub-filesets."""
 		return self._fileSets
 
-	def Files(self, fileType: FileType = FileTypes.Any, fileSet: Union[str, 'FileSet'] = None) -> Generator[File, None, None]:
-		if fileSet is None:
+	def Files(self, fileType: FileType = FileTypes.Any, fileSet: Union[bool, str, 'FileSet'] = None) -> Generator[File, None, None]:
+		"""
+		Method returning the files of this fileset.
+
+		:arg fileType: A filter for file types. Default: ``Any``.
+		:arg fileSet:  Specifies how to handle sub-filesets.
+		"""
+		if fileSet is False:
+			for file in self._files:
+				if (file.FileType in fileType):
+					yield file
+		elif fileSet is None:
 			for fileSet in self._fileSets.values():
 				for file in fileSet.Files(fileType):
 					yield file
 			for file in self._files:
 				if (file.FileType in fileType):
 					yield file
-
 		else:
 			if isinstance(fileSet, str):
+				fileSetName = fileSet
 				try:
-					fileSet = self._fileSets[fileSet]
+					fileSet = self._fileSets[fileSetName]
 				except KeyError as ex:
-					raise Exception("Fileset {name} not bound to fileset {fileset}.".format(name=fileSet.Name, fileset=self.Name)) from ex
+					raise Exception("Fileset {name} not bound to fileset {fileset}.".format(name=fileSetName, fileset=self.Name)) from ex
 			elif not isinstance(fileSet, FileSet):
 				raise TypeError("Parameter 'fileSet' is not of type 'str' or 'FileSet' nor value 'None'.")
 
@@ -621,21 +738,70 @@ class FileSet:
 				yield file
 
 	def AddFile(self, file: File) -> None:
+		"""
+		Method to add a single file to this fileset.
+
+		:arg file: A file to add to this fileset.
+		"""
 		self._files.append(file)
 		file._fileSet = self
 
 	def AddFiles(self, files: Iterable[File]) -> None:
+		"""
+		Method to add a multiple files to this fileset.
+
+		:arg files: An iterable of files to add to the fileset.
+		"""
 		for file in files:
 			self._files.append(file)
 			file._fileSet = self
 
-	def __getitem__(self, key):
+	def Validate(self):
+		"""Validate this fileset."""
+		if self._name is None or self._name == "":
+			raise Exception("Validation: FileSet has no name.")
+
+		if self._directory is None:
+			raise Exception(f"Validation: FileSet '{self._name}' has no directory.")
+		try:
+			path = self.ResolvedPath
+		except Exception as ex:
+			raise Exception(f"Validation: FileSet '{self._name}' could not compute resolved path.") from ex
+		if not path.exists():
+			raise Exception(f"Validation: FileSet '{self._name}'s directory '{path}' does not exist.")
+		if not path.is_dir():
+			raise Exception(f"Validation: FileSet '{self._name}'s directory '{path}' is not a directory.")
+
+		if self._design is None:
+			raise Exception(f"Validation: FileSet '{self._path}' has no design.")
+		if self._project is None:
+			raise Exception(f"Validation: FileSet '{self._path}' has no project.")
+
+		for fileSet in self._fileSets.values():
+			fileSet.Validate()
+		for file in self._files:
+			file.Validate()
+
+	def __len__(self):
+		"""Returns number of files incl. the files in the sub-filesets."""
+		fileCount = self._files.__len__()
+		for fileSet in self._fileSets:
+			fileCount += fileSet.__len__()
+
+		return fileCount
+
+	def __getitem__(self, key: Type[Attribute]):
+		"""Index access for returning attributes on this file."""
+		if not issubclass(key, Attribute):
+			raise TypeError("Parameter 'key' is not an 'Attribute'.")
+
 		try:
 			return self._attributes[key]
 		except KeyError:
-			return self._fileSet[key]
+			return key.resolve(self, key)
 
-	def __setitem__(self, key, value):
+	def __setitem__(self, key: Type[Attribute], value: typing_Any):
+		"""Index access for setting attributes on this file."""
 		self._attributes[key] = value
 
 	def GetOrCreateVHDLLibrary(self, name):
@@ -648,6 +814,7 @@ class FileSet:
 
 	@property
 	def VHDLLibrary(self) -> 'VHDLLibrary':
+		"""Property setting or returning the VHDL library of this fileset."""
 		if self._vhdlLibrary is not None:
 			return self._vhdlLibrary
 		elif self._parent is not None:
@@ -663,6 +830,7 @@ class FileSet:
 
 	@property
 	def VHDLVersion(self) -> VHDLVersion:
+		"""Property setting or returning the VHDL version of this fileset."""
 		if self._vhdlVersion is not None:
 			return self._vhdlVersion
 		elif self._parent is not None:
@@ -678,6 +846,7 @@ class FileSet:
 
 	@property
 	def VerilogVersion(self) -> VerilogVersion:
+		"""Property setting or returning the Verilog version of this fileset."""
 		if self._verilogVersion is not None:
 			return self._verilogVersion
 		elif self._parent is not None:
@@ -693,6 +862,7 @@ class FileSet:
 
 	@property
 	def SVVersion(self) -> SystemVerilogVersion:
+		"""Property setting or returning the SystemVerilog version of this fileset."""
 		if self._svVersion is not None:
 			return self._svVersion
 		elif self._parent is not None:
@@ -707,6 +877,7 @@ class FileSet:
 		self._svVersion = value
 
 	def __str__(self):
+		"""Returns the fileset's name."""
 		return self._name
 
 
@@ -715,9 +886,10 @@ class VHDLLibrary:
 	"""
 	A :term:`VHDLLibrary` represents a group of VHDL source files compiled into the same VHDL library.
 
-	:arg name:      The VHDL libraries' name.
-	:arg project:   Project the VHDL library is associated with.
-	:arg design:    Design the VHDL library is associated with.
+	:arg name:        The VHDL libraries' name.
+	:arg project:     Project the VHDL library is associated with.
+	:arg design:      Design the VHDL library is associated with.
+	:arg vhdlVersion: Default VHDL version for files in this VHDL library, if not specified for the file itself.
 	"""
 
 	_name:        str
@@ -752,6 +924,7 @@ class VHDLLibrary:
 
 	@property
 	def Project(self) -> Nullable['Project']:
+		"""Property setting or returning the project this VHDL library is used in."""
 		return self._project
 
 	@Project.setter
@@ -760,6 +933,7 @@ class VHDLLibrary:
 
 	@property
 	def Design(self) -> Nullable['Design']:
+		"""Property setting or returning the design this VHDL library is used in."""
 		return self._design
 
 	@Design.setter
@@ -775,21 +949,26 @@ class VHDLLibrary:
 
 	@property
 	def Files(self) -> Generator[File, None, None]:
+		"""Read-only property to return all files in this VHDL library."""
 		for file in self._files:
 			yield file
 
 	@property
 	def VHDLVersion(self) -> VHDLVersion:
+		"""Property setting or returning the VHDL version of this VHDL library."""
 		if self._vhdlVersion is not None:
 			return self._vhdlVersion
-		else:
+		elif self._design is not None:
 			return self._design.VHDLVersion
+		else:
+			raise Exception("VHDLVersion is not set on VHDLLibrary nor parent object.")
 
 	@VHDLVersion.setter
 	def VHDLVersion(self, value: VHDLVersion) -> None:
 		self._vhdlVersion = value
 
 	def __str__(self):
+		"""Returns the VHDL library's name."""
 		return self._name
 
 
@@ -802,18 +981,22 @@ class Design:
 	designs with VHDL source files, a independent `VHDLLibraries` overlay structure
 	exists.
 
-	:arg name:      The design's name.
-	:arg directory: Path of this design (absolute or relative to the project).
-	:arg project:   Project the design is associated with.
+	:arg name:            The design's name.
+	:arg topLevel:        Name of the design's toplevel.
+	:arg directory:       Path of this design (absolute or relative to the project).
+	:arg project:         Project the design is associated with.
+	:arg vhdlVersion:     Default VHDL version for files in this design, if not specified for the file itself.
+	:arg verilogVersion:  Default Verilog version for files in this design, if not specified for the file itself.
+	:arg svVersion:       Default SystemVerilog version for files in this design, if not specified for the file itself.
 	"""
 
 	_name:                  str
 	_topLevel:              Nullable[str]
 	_project:               Nullable['Project']
-	_directory:             Nullable[Path]
+	_directory:             pathlib_Path
 	_fileSets:              Dict[str, FileSet]
 	_defaultFileSet:        Nullable[FileSet]
-	_attributes:            Dict[Attribute, typing_Any]
+	_attributes:            Dict[Type[Attribute], typing_Any]
 
 	_vhdlLibraries:         Dict[str, VHDLLibrary]
 	_vhdlVersion:           VHDLVersion
@@ -825,7 +1008,7 @@ class Design:
 		self,
 		name: str,
 		topLevel: str = None,
-		directory: Path = Path("."),
+		directory: pathlib_Path = pathlib_Path("."),
 		project: 'Project' = None,
 		vhdlVersion: VHDLVersion = None,
 		verilogVersion: VerilogVersion = None,
@@ -848,6 +1031,7 @@ class Design:
 
 	@property
 	def Name(self) -> str:
+		"""Property setting or returning the design's name."""
 		return self._name
 
 	@Name.setter
@@ -856,6 +1040,7 @@ class Design:
 
 	@property
 	def TopLevel(self) -> str:
+		"""Property setting or returning the fileset's toplevel."""
 		return self._topLevel
 
 	@TopLevel.setter
@@ -864,6 +1049,7 @@ class Design:
 
 	@property
 	def Project(self) -> Nullable['Project']:
+		"""Property setting or returning the project this design is used in."""
 		return self._project
 
 	@Project.setter
@@ -871,15 +1057,17 @@ class Design:
 		self._project = value
 
 	@property
-	def Directory(self) -> Path:
+	def Directory(self) -> pathlib_Path:
+		"""Property setting or returning the directory this design is located in."""
 		return self._directory
 
 	@Directory.setter
-	def Directory(self, value: Path) -> None:
+	def Directory(self, value: pathlib_Path) -> None:
 		self._directory = value
 
 	@property
-	def ResolvedPath(self) -> Path:
+	def ResolvedPath(self) -> pathlib_Path:
+		"""Read-only property returning the resolved path of this fileset."""
 		if self._directory.is_absolute():
 			return self._directory.resolve()
 		elif self._project is not None:
@@ -888,13 +1076,15 @@ class Design:
 			if path.is_absolute():
 				return path
 			else:
-				return path.relative_to(Path.cwd())
+				# WORKAROUND: https://stackoverflow.com/questions/67452690/pathlib-path-relative-to-vs-os-path-relpath
+				return pathlib_Path(path_relpath(path, pathlib_Path.cwd()))
 		else:
 			# TODO: message and exception type
 			raise Exception("")
 
 	@property
 	def DefaultFileSet(self) -> FileSet:
+		"""Property setting or returning the default fileset of this design."""
 		return self._defaultFileSet
 
 	@DefaultFileSet.setter
@@ -915,9 +1105,16 @@ class Design:
 	# TODO: return generator with another method
 	@property
 	def FileSets(self) -> Dict[str, FileSet]:
+		"""Read-only property returning the a dictionary of filesets."""
 		return self._fileSets
 
 	def Files(self, fileType: FileType = FileTypes.Any, fileSet: Union[str, FileSet] = None) -> Generator[File, None, None]:
+		"""
+		Method returning the files of this design.
+
+		:arg fileType: A filter for file types. Default: ``Any``.
+		:arg fileSet:  Specifies if all files from all filesets (``fileSet=None``) are files from a single fileset are returned.
+		"""
 		if fileSet is None:
 			for fileSet in self._fileSets.values():
 				for file in fileSet.Files(fileType):
@@ -934,13 +1131,48 @@ class Design:
 			for file in fileSet.Files(fileType):
 				yield file
 
-	def __getitem__(self, key):
+	def Validate(self):
+		"""Validate this design."""
+		if self._name is None or self._name == "":
+			raise Exception("Validation: Design has no name.")
+
+		if self._directory is None:
+			raise Exception(f"Validation: Design '{self._name}' has no directory.")
+		try:
+			path = self.ResolvedPath
+		except Exception as ex:
+			raise Exception(f"Validation: Design '{self._name}' could not compute resolved path.") from ex
+		if not path.exists():
+			raise Exception(f"Validation: Design '{self._name}'s directory '{path}' does not exist.")
+		if not path.is_dir():
+			raise Exception(f"Validation: Design '{self._name}'s directory '{path}' is not a directory.")
+
+		if len(self._fileSets) == 0:
+			raise Exception(f"Validation: Design '{self._name}' has no fileset.")
+		try:
+			if self._defaultFileSet is not self._fileSets[self._defaultFileSet.Name]:
+				raise Exception(f"Validation: Design '{self._name}'s default fileset is the same as listed in filesets.")
+		except KeyError as ex:
+			raise Exception(f"Validation: Design '{self._name}'s default fileset is not in list of filesets.") from ex
+		if self._project is None:
+			raise Exception(f"Validation: Design '{self._path}' has no project.")
+
+		for fileSet in self._fileSets.values():
+			fileSet.Validate()
+
+	def __len__(self):
+		return self._fileSets.__len__()
+
+	def __getitem__(self, key: Type[Attribute]):
+		if not issubclass(key, Attribute):
+			raise TypeError("Parameter 'key' is not an 'Attribute'.")
+
 		try:
 			return self._attributes[key]
 		except KeyError:
-			return self._fileSet[key]
+			return key.resolve(self, key)
 
-	def __setitem__(self, key, value):
+	def __setitem__(self, key: Type[Attribute], value: typing_Any):
 		self._attributes[key] = value
 
 	@property
@@ -1015,6 +1247,13 @@ class Design:
 		for file in files:
 			self.AddFile(file)
 
+	def AddVHDLLibrary(self, vhdlLibrary: VHDLLibrary):
+		if vhdlLibrary.Name in self._vhdlLibraries:
+			if self._vhdlLibraries[vhdlLibrary.Name] is vhdlLibrary:
+				raise Exception(f"The VHDLLibrary '{vhdlLibrary.Name}' was already added to the design.")
+			else:
+				raise Exception(f"A VHDLLibrary with same name ('{vhdlLibrary.Name}') already exists for this design.")
+
 	def __str__(self):
 		return self._name
 
@@ -1024,15 +1263,18 @@ class Project:
 	"""
 	A :term:`Project` represents a group of designs and the source files therein.
 
-	:arg name:          The project's name.
-	:arg rootDirectory: Base-path to the project.
+	:arg name:            The project's name.
+	:arg rootDirectory:   Base-path to the project.
+	:arg vhdlVersion:     Default VHDL version for files in this project, if not specified for the file itself.
+	:arg verilogVersion:  Default Verilog version for files in this project, if not specified for the file itself.
+	:arg svVersion:       Default SystemVerilog version for files in this project, if not specified for the file itself.
 	"""
 
 	_name:            str
-	_rootDirectory:   Nullable[Path]
+	_rootDirectory:   pathlib_Path
 	_designs:         Dict[str, Design]
 	_defaultDesign:   Design
-	_attributes:      Dict[Attribute, typing_Any]
+	_attributes:      Dict[Type[Attribute], typing_Any]
 
 	_vhdlVersion:     VHDLVersion
 	_verilogVersion:  VerilogVersion
@@ -1041,7 +1283,7 @@ class Project:
 	def __init__(
 		self,
 		name: str,
-		rootDirectory: Path = Path("."),
+		rootDirectory: pathlib_Path = pathlib_Path("."),
 		vhdlVersion: VHDLVersion = None,
 		verilogVersion: VerilogVersion = None,
 		svVersion: SystemVerilogVersion = None
@@ -1057,23 +1299,27 @@ class Project:
 
 	@property
 	def Name(self) -> str:
+		"""Property setting or returning the project's name."""
 		return self._name
 
 	@property
-	def RootDirectory(self) -> Path:
+	def RootDirectory(self) -> pathlib_Path:
+		"""Property setting or returning the root directory this project is located in."""
 		return self._rootDirectory
 
 	@RootDirectory.setter
-	def RootDirectory(self, value: Path) -> None:
+	def RootDirectory(self, value: pathlib_Path) -> None:
 		self._rootDirectory = value
 
 	@property
-	def ResolvedPath(self) -> Path:
+	def ResolvedPath(self) -> pathlib_Path:
+		"""Read-only property returning the resolved path of this fileset."""
 		path = self._rootDirectory.resolve()
 		if self._rootDirectory.is_absolute():
 			return path
 		else:
-			return path.relative_to(Path.cwd())
+			# WORKAROUND: https://stackoverflow.com/questions/67452690/pathlib-path-relative-to-vs-os-path-relpath
+			return pathlib_Path(path_relpath(path, pathlib_Path.cwd()))
 
 	# TODO: return generator with another method
 	@property
@@ -1084,13 +1330,46 @@ class Project:
 	def DefaultDesign(self) -> Design:
 		return self._defaultDesign
 
-	def __getitem__(self, key):
+	def Validate(self):
+		"""Validate this project."""
+		if self._name is None or self._name == "":
+			raise Exception("Validation: Project has no name.")
+
+		if self._rootDirectory is None:
+			raise Exception(f"Validation: Project '{self._name}' has no root directory.")
+		try:
+			path = self.ResolvedPath
+		except Exception as ex:
+			raise Exception(f"Validation: Project '{self._name}' could not compute resolved path.") from ex
+		if not path.exists():
+			raise Exception(f"Validation: Project '{self._name}'s directory '{path}' does not exist.")
+		if not path.is_dir():
+			raise Exception(f"Validation: Project '{self._name}'s directory '{path}' is not a directory.")
+
+		if len(self._designs) == 0:
+			raise Exception(f"Validation: Project '{self._name}' has no design.")
+		try:
+			if self._defaultDesign is not self._designs[self._defaultDesign.Name]:
+				raise Exception(f"Validation: Project '{self._name}'s default design is the same as listed in designs.")
+		except KeyError as ex:
+			raise Exception(f"Validation: Project '{self._name}'s default design is not in list of designs.") from ex
+
+		for design in self._designs.values():
+			design.Validate()
+
+	def __len__(self):
+		return self._designs.__len__()
+
+	def __getitem__(self, key: Type[Attribute]):
+		if not issubclass(key, Attribute):
+			raise TypeError("Parameter 'key' is not an 'Attribute'.")
+
 		try:
 			return self._attributes[key]
 		except KeyError:
-			return self._fileSet[key]
+			return key.resolve(self, key)
 
-	def __setitem__(self, key, value):
+	def __setitem__(self, key: Type[Attribute], value: typing_Any):
 		self._attributes[key] = value
 
 	@property
